@@ -56,30 +56,7 @@ public:
     }
 
 protected:
-    bool checkLayer(Layer* l) {
-        /* Search for Element whose bounding box center is closest to the place (x,y) where the object is searched for.
-         * Only those strokes are taken into account that pass the appropriate checkElement test.
-         */
-        bool found = false;
-        double minDistSq = std::numeric_limits<double>::max();
-        for (Element* e: l->getElements()) {
-            const double eX = e->getX() + e->getElementWidth() / 2.0;
-            const double eY = e->getY() + e->getElementHeight() / 2.0;
-            const double dx = eX - this->x;
-            const double dy = eY - this->y;
-            const double distSq = dx * dx + dy * dy;
-            const GdkRectangle matchRect = {gint(x - 10), gint(y - 10), 20, 20};
-            if (e->intersectsArea(&matchRect) && distSq < minDistSq) {
-                if (this->checkElement(e)) {
-                    minDistSq = distSq;
-                    found = true;
-                }
-            }
-        }
-        return found;
-    }
-
-    virtual bool checkElement(Element* e) = 0;
+    virtual bool checkLayer(const Layer* l) = 0;
 
 protected:
     XojPageView* view{};
@@ -89,52 +66,48 @@ protected:
 
 class SelectObject: public BaseSelectObject {
 public:
-    SelectObject(XojPageView* view):
-            BaseSelectObject(view), strokeMatch(nullptr), elementMatch(nullptr), gap(1000000000) {}
+    SelectObject(XojPageView* view): BaseSelectObject(view), match(nullptr) {}
 
     ~SelectObject() override = default;
 
     bool at(double x, double y, bool multiLayer = false) override {
         BaseSelectObject::at(x, y, multiLayer);
 
-        if (strokeMatch) {
-            elementMatch = strokeMatch;
-        }
-
-        if (elementMatch) {
-            view->xournal->setSelection(new EditSelection(view->xournal->getControl()->getUndoRedoHandler(),
-                                                          elementMatch, view, view->page));
-
+        if (match) {
+            view->xournal->setSelection(
+                    new EditSelection(view->xournal->getControl()->getUndoRedoHandler(), match, view, view->page));
             view->repaintPage();
-
             return true;
         }
 
         return false;
     }
 
-protected:
-    bool checkElement(Element* e) override {
-        if (e->getType() == ELEMENT_STROKE) {
-            Stroke* s = (Stroke*)e;
-            double tmpGap = 0;
-            if ((s->intersects(x, y, 5, &tmpGap)) && (gap > tmpGap)) {
-                gap = tmpGap;
-                strokeMatch = s;
-                return true;
-            }
-        } else {
-            elementMatch = e;
-            return true;
-        }
+    static constexpr double ACTION_RADIUS = 5.;
 
-        return false;
+    bool checkLayer(const Layer* l) override {
+        double minDistance = ACTION_RADIUS;
+        // Iterate starting from the front-most element
+        for (auto it = l->getElements().rbegin(); it < l->getElements().rend(); ++it) {
+            // First perform a rough check to avoid expensive calls to Stroke::distanceTo()
+            if ((*it)->intersectsArea(x - minDistance, y - minDistance, 2. * minDistance, 2. * minDistance)) {
+                double d = (*it)->distanceTo(x, y);
+                if (d == 0.0) {
+                    this->match = *it;
+                    return true;
+                }
+                if (d < minDistance) {
+                    this->match = *it;
+                    minDistance = d;
+                    // Keep going, we may find something closer
+                }
+            }
+        }
+        return minDistance != ACTION_RADIUS;
     }
 
 private:
-    Stroke* strokeMatch;
-    Element* elementMatch;
-    double gap;
+    Element* match;
 };
 
 class PlayObject: public BaseSelectObject {
@@ -154,32 +127,44 @@ public:
     bool at(double x, double y, bool multiLayer = false) override { return BaseSelectObject::at(x, y, multiLayer); }
 
 protected:
-    bool checkElement(Element* e) override {
-        if (e->getType() != ELEMENT_STROKE && e->getType() != ELEMENT_TEXT) {
-            return false;
-        }
+    static constexpr double ACTION_RADIUS = 15.;
 
-        AudioElement* s = (AudioElement*)e;
-        double tmpGap = 0;
-        if ((s->intersects(x, y, 15, &tmpGap))) {
-            size_t ts = s->getTimestamp();
-
-            if (auto fn = s->getAudioFilename(); !fn.empty()) {
-                if (!fn.has_parent_path() || fs::weakly_canonical(fn.parent_path()) == "/") {
-                    auto const& path = view->settings->getAudioFolder();
-                    // Assume path exists
-                    fn = path / fn;
+    /// Plays every element of the layer that are closer than ACTION_RADIUS
+    bool checkLayer(const Layer* l) override {
+        bool found = false;
+        for (auto&& e: l->getElements()) {
+            if (auto* audio = dynamic_cast<AudioElement*>(e); audio) {
+                // First perform a rough check to avoid expensive calls to Stroke::distanceTo()
+                if (audio->intersectsArea(x - ACTION_RADIUS, y - ACTION_RADIUS, 2. * ACTION_RADIUS,
+                                          2. * ACTION_RADIUS)) {
+                    double d = audio->distanceTo(x, y);
+                    if (d < ACTION_RADIUS) {
+                        found = playElement(audio) || found;
+                    }
                 }
-                auto* audioController = view->getXournal()->getControl()->getAudioController();
-                if (!audioController) {
-                    g_warning("Audio has been disabled");
-                    return false;
-                }
-                bool success = audioController->startPlayback(fn, (unsigned int)ts);
-                playbackStatus = {success, std::move(fn)};
-                return success;
             }
         }
+        return found;
+    }
+
+    bool playElement(const AudioElement* s) {
+        size_t ts = s->getTimestamp();
+
+        if (auto fn = s->getAudioFilename(); !fn.empty()) {
+            if (!fn.has_parent_path() || fs::weakly_canonical(fn.parent_path()) == "/") {
+                auto const& path = view->settings->getAudioFolder();
+                // Assume path exists
+                fn = path / fn;
+            }
+            auto* audioController = view->getXournal()->getControl()->getAudioController();
+            if (!audioController) {
+                g_warning("Audio has been disabled");
+                return false;
+            }
+            bool success = audioController->startPlayback(fn, (unsigned int)ts);
+            playbackStatus = {success, std::move(fn)};
+            return success;
+        }
         return false;
     }
 };
