@@ -2,9 +2,11 @@
 
 #include <algorithm>  // for max, min
 #include <cassert>    // for assert
-#include <cstdio>     // for snprintf
+#include <iomanip>    // for operator<<, setfill, setw
 #include <sstream>    // for operator<<, stringstream, basic_ostream, char_t...
 
+#include "util/serdesstream.h"  // for serdes_stream
+
 float Util::as_grayscale_color(Color color) {
     GdkRGBA components = rgb_to_GdkRGBA(color);
     float componentAvg = static_cast<float>(components.red + components.green + components.blue) / 3.0f;
@@ -20,12 +22,10 @@ float Util::get_color_contrast(Color color1, Color color2) {
 }
 
 auto Util::rgb_to_hex_string(Color rgb) -> std::string {
-    char resultHex[7];
+    auto s = serdes_stream<std::ostringstream>();
+    auto tmp_color = rgb;
+    tmp_color.alpha = 0;
+    s << "#" << std::hex << std::setfill('0') << std::setw(6) << std::right << tmp_color;
 
-    // 06: Disregard alpha channel and pad with zeroes to a length of 6.
-    assert(std::snprintf(resultHex, 7, "%06x", uint32_t(rgb) & 0xffffff) > 0);
-
-    std::stringstream result;
-    result << "#" << resultHex;
-    return result.str();
+    return s.str();
 }
